/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             List Manager
*  SCOPE :              Implements list management
*  FILE :               listgen.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) Eric Abecassis, 1994
*  HISTORY :
*
******************************************************************************/

/* IMPORTS *******************************************************************/

#ifdef __GNUG__
#pragma interface
#endif

#ifndef NULL
#define NULL 0
#endif

/*=
Module Description
This module implements manipulation of the list.
Note that first element of a list is in position 0.
Insert an element at the end of a list is done by inserting in position -1.
In an iterator construction, max = -1 set scan to be done until the end of the
list.
=*/

#define List VR_List

class ListCell {
  public :
  void* elem ;
  ListCell *next ;
  ListCell(void* el) : elem(el) {} ;
} ;

class List {
friend class IterList ;
  public :

  // create an empty list
  List() : first(NULL), size(0) {} ;
  
  // remove element at the position pos
  void* remove(int pos) ; 

  // return element at the position pos.
  ListCell* get(int pos) const ;

  // return current size of the list
  int count() const { return size ; } 

  // Move content of the receiver into parameter
  void capture(List&) ;

  protected :

  // head and tail of the list
  ListCell *first ;
  int size ;
  // insert element in position pos
  void insert(void* el, int pos) ;
} ;


class IterList {
  public :

  // Build an iterator between position min and max
  IterList(List& l, int min, int max) ;

  // Move cursor to the next element
  void next() ;

  // Return the current element
  void* get() { return curr == NULL ? NULL : curr->elem ; } ;

  // Return TRUE if there is more elements to scan
  int more() { return (cpos <= end) ; } ;

  // Reset the scan between min and max. By default, first to last
  void reset(int min=0, int max = -1) ;

  // Remove the current element
  void* remove() ;

  // Insert after current position
  void insert(void* el) ;
    
  private :

  List* myList ;
  ListCell* prev ;
  ListCell* curr ;
  int cpos, start, end ;
} ;

